@@ -137,6 +137,15 @@ setClass("niftiExtension",
          prototype(extensions=list()),
          contains="nifti")
 
+#############################################################################
+## setClass("niftiAuditTrail")
+#############################################################################
+
+setClass("niftiAuditTrail",
+         representation(trail="ANY"),
+         prototype(trail=newAuditTrail()),
+         contains="niftiExtension")
+
 #############################################################################
 ## setClass("niftiExtensionSection")
 #############################################################################
